using System;
using System.Collections.Generic;
using System.Data;
using System.Text;

namespace AutoBroker
{
    public class SimpleAgent : IAgent
    {
        public const float ToleranceTrigger = 0.2f;        
        private static IValues fixingValues = null;
        private static PastToNowRatioIndicator beforeRatio1 = null;
        private static PastToNowRatioIndicator beforeRatio3 = null;
        private static PastToNowRatioIndicator beforeRatio5 = null;
        private static PastToNowRatioIndicator beforeRatio10 = null;
        private static SmaIndicator sma15 = null;
        private static SmaIndicator sma30 = null;
        private static SmaIndicator sma45 = null;

        private AgentType myType;
        
        private long usageCount = 0;

        private float goodRatio1 = 1.0f;
        private float goodRatio3 = 1.0f;
        private float goodRatio5 = 1.0f;
        private float goodRatio10 = 1.0f;

        private float goodSMA15 = 1.0f;
        private float goodSMA30 = 1.0f;
        private float goodSMA45 = 1.0f;

        public long GetUsageCount()
        {
            return usageCount;
        }

        public static void SetFixingValues(IValues values)
        {
            fixingValues = values;
            beforeRatio1 = new PastToNowRatioIndicator(1, values);
            beforeRatio3 = new PastToNowRatioIndicator(3, values);
            beforeRatio5 = new PastToNowRatioIndicator(5, values);
            beforeRatio10 = new PastToNowRatioIndicator(10, values);
            sma15 = new SmaIndicator(15, values);
            sma30 = new SmaIndicator(30, values);
            sma45 = new SmaIndicator(45, values);
        }

        private static bool IsWithinTolerance(float baseValue, float secondValue)
        {
            if (secondValue==0.0f) secondValue = 0.0001f;
            return Math.Abs((baseValue / secondValue) - 1.0f) < ToleranceTrigger;
        }

        public AgentDecision GetDecision(int fixingIndex)
        {
            float fixingVal = fixingValues.GetValue(fixingIndex);
            float pastRatio1 = beforeRatio1.GetValue(fixingIndex);
            float pastRatio3 = beforeRatio3.GetValue(fixingIndex);
            float pastRatio5 = beforeRatio5.GetValue(fixingIndex);
            float pastRatio10 = beforeRatio10.GetValue(fixingIndex);
            if (float.IsNaN(fixingVal) || float.IsNaN(pastRatio1) || float.IsNaN(pastRatio3) || float.IsNaN(pastRatio10))
            {
                return AgentDecision.Wait;
            }
            int matchingRules = 0;
            if (IsWithinTolerance(goodRatio1, pastRatio1)) matchingRules++;
            if (IsWithinTolerance(goodRatio3, pastRatio3)) matchingRules++;
            if (IsWithinTolerance(goodRatio5, pastRatio5)) matchingRules++;
            if (IsWithinTolerance(goodRatio10, pastRatio10)) matchingRules++;
            if (IsWithinTolerance(goodSMA15, fixingVal / sma15.GetValue(fixingIndex))) matchingRules++;
            if (IsWithinTolerance(goodSMA30, fixingVal / sma30.GetValue(fixingIndex))) matchingRules++;
            if (IsWithinTolerance(goodSMA45, fixingVal / sma45.GetValue(fixingIndex))) matchingRules++;
            if (matchingRules > 6)
            {
                if (myType == AgentType.Buyer) return AgentDecision.BuyRapidly;
                if (myType == AgentType.Seller) return AgentDecision.SellRapidly;
                usageCount++;
            }
            else
            if (matchingRules >4)
            {
                if (myType == AgentType.Buyer) return AgentDecision.BuySlowly;
                if (myType == AgentType.Seller) return AgentDecision.SellSlowly;
                usageCount++;
            }
            return AgentDecision.Wait;
        }

        public void SetAgentData(AgentType agentType, int fixingIndex)
        {
            float fixingVal = fixingValues.GetValue(fixingIndex);
            myType = agentType;
            goodRatio1 = beforeRatio1.GetValue(fixingIndex);
            goodRatio3 = beforeRatio3.GetValue(fixingIndex);
            goodRatio5 = beforeRatio5.GetValue(fixingIndex);
            goodRatio10 = beforeRatio10.GetValue(fixingIndex);
            goodSMA15 = fixingVal / sma15.GetValue(fixingIndex);
            goodSMA30 = fixingVal / sma30.GetValue(fixingIndex);
            goodSMA45 = fixingVal / sma45.GetValue(fixingIndex);
        }

    }
}
