using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using System.Net;
using System.Net.Sockets;
using System.IO;

namespace AutoBroker
{
    public partial class MainForm : Form
    {
        public Stocks LoadedStocks = null;

        public MainForm()
        {
            InitializeComponent();
            mainInstance = this;
        }
        private static MainForm mainInstance = null;

        public static MainForm GetInstance()
        {
            if (mainInstance == null)
            {
                mainInstance = new MainForm();
            }
            return mainInstance;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            LoadData();
        }

        private void LoadData()
        {
            comboStocks.Items.Clear();
            LoadedStocks = new Stocks(System.Environment.CurrentDirectory + @"\notowania");
            foreach (StockInfo stock in LoadedStocks.stockInfo)
            {
                comboStocks.Items.Add(stock.GetFullName());
            }
            UpdateStatsWindow();
        }

        public StockInfo GetStockInfoForName(string name)
        {
            if (name == null) return null;
            foreach (StockInfo stock in LoadedStocks.stockInfo)
            {
                if (stock.GetFullName().ToLower() == name.ToLower())
                {
                    return stock;
                }
            }
            return null;
        }

        private void comboStocks_SelectedIndexChanged(object sender, EventArgs e)
        {
            chartControl.Chart = new StockChartValues(GetStockInfoForName(comboStocks.Text));
            chartControl.RefreshDataVisualization();
            StockInfo curStock = GetStockInfoForName(comboStocks.Text);
            if (chartControl.Chart != null)
            {
                float min, max;
                chartControl.Chart.GetMinMaxValue(out min, out max);
                labelPriceMin.Text = min.ToString("F2") + " z";
                labelPriceMax.Text = max.ToString("F2") + " z";
            }
            if (curStock!=null)
            {
                if (curStock.Fixing.Count > 0)
                {
                    labelFirstFixing.Text = curStock.Fixing[0].Date.ToShortDateString();
                }
            }
            UpdateStatsWindow();
        }

        private void chartControl_Load(object sender, EventArgs e)
        {
        }

        private void zakoczToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void pobierzNotowaniaToolStripMenuItem_Click(object sender, EventArgs e)
        {
            bool downloadSuccess = false;
            string errorMessage = "";
            string downloadFile = System.Environment.CurrentDirectory + 
                @"\pobrane\parkiet" + DateTime.Now.ToShortDateString() + ".zip";
            for (int attempt = 0; attempt < 5; attempt++)
            {
                try
                {
                    WebClient InetClient = new WebClient();                    
                    InetClient.UseDefaultCredentials = true;                    
                    InetClient.DownloadFile(@"http://www.parkiet.com/dane/danesesji/bazytxt/akcje/akcje.zip", downloadFile);
                    downloadSuccess = true;
                }
                catch (Exception ex)
                {
                    errorMessage = ex.Message;
                }
                if (downloadSuccess)
                {
                    break;
                }
            }

            if (!downloadSuccess)
            {
                MessageBox.Show("Nastpi bd przy pobieraniu: " + errorMessage);
                return;
            }
            string unzipArgs = "-o -qq " + "\"" + downloadFile + "\"" + " -d " +
                                "\"" +  System.Environment.CurrentDirectory + @"\notowania" + "\"";
            System.Diagnostics.ProcessStartInfo startInfo = new System.Diagnostics.ProcessStartInfo(System.Environment.CurrentDirectory + @"\narzedzia\unzip.exe");
            startInfo.Arguments = unzipArgs;
            startInfo.UseShellExecute = false;
            startInfo.CreateNoWindow = true;
            //startInfo.RedirectStandardOutput = true;
            
            System.Diagnostics.Process newProcess = System.Diagnostics.Process.Start(startInfo);
            newProcess.WaitForExit();     

            LoadData();

        }

        private void tabStats_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateStatsWindow();
        }

        private void UpdateStatsWindow()
        {
            if (tabStats.SelectedTab == tabPageMain)
            {
                PrepareStatisticsMain();
            }
            if (tabStats.SelectedTab == tabPageTrend)
            {
                PrepareStatisticsTrend();
            }
            if (tabStats.SelectedTab == tabPageWeek)
            {
                PrepareStatisticsWeek();
            }
        }

        public static ListViewItem GetTextListItem(string text, string value)
        {
            ListViewItem listItem = new ListViewItem(text);
            listItem.SubItems.Add(new ListViewItem.ListViewSubItem(listItem,value));
            return listItem;
        }

        private void PrepareStatisticsMain()
        {
            listViewStats.Items.Clear();
            StockInfo curStock = GetStockInfoForName(comboStocks.Text);
            if (curStock != null)
            {
                StockStats stats = curStock.GetStats();
                listViewStats.Items.Add(GetTextListItem("Ilo wzrostw", stats.StocksUpCount.ToString()));
                listViewStats.Items.Add(GetTextListItem("Ilo spadkw", stats.StocksDownCount.ToString()));
                listViewStats.Items.Add(GetTextListItem("redni wzrost", (stats.AverageUpPercent*100.0f).ToString("F2") + " %"));
                listViewStats.Items.Add(GetTextListItem("redni spadek", (stats.AverageDownPercent * 100.0f).ToString("F2") + " %"));
                listViewStats.Items.Add(GetTextListItem("Maks. wzrost", (stats.MaxStockUpPercent * 100.0f).ToString("F2") + " %"));
                listViewStats.Items.Add(GetTextListItem("Maks. spadek", (stats.MaxStockDownPercent * 100.0f).ToString("F2") + " %"));
                listViewStats.Items.Add(GetTextListItem("Ilo notowa", stats.TotalFixings.ToString()));
            }
            else
            {
                listViewStats.Items.Add(GetTextListItem("Ilo wzrostw", "---"));
                listViewStats.Items.Add(GetTextListItem("Ilo spadkw", "---"));
                listViewStats.Items.Add(GetTextListItem("redni wzrost", "---"));
                listViewStats.Items.Add(GetTextListItem("redni spadek", "---"));
                listViewStats.Items.Add(GetTextListItem("Maks. wzrost", "---"));
                listViewStats.Items.Add(GetTextListItem("Maks. spadek", "---"));
                listViewStats.Items.Add(GetTextListItem("Ilo notowa", "---"));
            }
        }

        private void PrepareStatisticsTrend()
        {
            listViewStats.Items.Clear();
            StockInfo curStock = GetStockInfoForName(comboStocks.Text);
            if (curStock != null)
            {
                StockStats stats = curStock.GetStats();
                listViewStats.Items.Add(GetTextListItem("Najd. wznoszcy", stats.LongestUpTrend.ToString()));
                listViewStats.Items.Add(GetTextListItem("Najd. spadkowy", stats.LongestDownTrend.ToString()));                
                listViewStats.Items.Add(GetTextListItem("r. dug. wznosz.", stats.AverageUpTrendLen.ToString()));
                listViewStats.Items.Add(GetTextListItem("r. dug. spad.", stats.AverageDownTrendLen.ToString()));
            }
            else
            {
                listViewStats.Items.Add(GetTextListItem("Najd. wznoszcy", "---"));
                listViewStats.Items.Add(GetTextListItem("Najd. spadkowy", "---"));
                listViewStats.Items.Add(GetTextListItem("r. dug. wznosz.", "---"));
                listViewStats.Items.Add(GetTextListItem("r. dug. spad.", "---"));
            }
        }

        private void PrepareStatisticsWeek()
        {
            listViewStats.Items.Clear();
            StockInfo curStock = GetStockInfoForName(comboStocks.Text);
            if (curStock != null)
            {
                StockStats stats = curStock.GetStats();
                listViewStats.Items.Add("rednie zmiany:");
                listViewStats.Items.Add(GetTextListItem("Poniedziaek", (stats.AverageDayPercent[1] * 100.0f).ToString("F2") + " %"));
                listViewStats.Items.Add(GetTextListItem("Wtorek", (stats.AverageDayPercent[2] * 100.0f).ToString("F2") + " %"));
                listViewStats.Items.Add(GetTextListItem("roda", (stats.AverageDayPercent[3] * 100.0f).ToString("F2") + " %"));
                listViewStats.Items.Add(GetTextListItem("Czwartek", (stats.AverageDayPercent[4] * 100.0f).ToString("F2") + " %"));
                listViewStats.Items.Add(GetTextListItem("Pitek", (stats.AverageDayPercent[5] * 100.0f).ToString("F2") + " %"));
            }
            else
            {
                listViewStats.Items.Add("rednie zmiany:");
                listViewStats.Items.Add(GetTextListItem("Poniedziaek", "---"));
                listViewStats.Items.Add(GetTextListItem("Wtorek", "---"));
                listViewStats.Items.Add(GetTextListItem("roda", "---"));
                listViewStats.Items.Add(GetTextListItem("Czwartek", "---"));
                listViewStats.Items.Add(GetTextListItem("Pitek", "---"));
            }
        }              

        private void odchylenieOdTrenduToolStripMenuItem_Click(object sender, EventArgs e)
        {
            TrendRatioTypicalReaction.PerformAnalysis();
        }

        private void systemWieloagentowyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            StockInfo curStock = GetStockInfoForName(comboStocks.Text);
            if (curStock != null)
            {
                StreamWriter writer = File.CreateText(System.Environment.CurrentDirectory + @"\analizy\symulacja"+curStock.Name+".rtf");
                writer.WriteLine("Multiagents");
                writer.WriteLine();

                AgentManager agMan = new AgentManager();
                agMan.PerformSimulation(new StockChartValues(curStock), writer);
                writer.Close();
            }
        }

        private void transakcjeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            StockCartForm cartWindow = new StockCartForm();
            cartWindow.LoadFromFile("portfele/tm.txt");
            cartWindow.Show();
        }

        private void eksportujDoAnalizy_Click(object sender, EventArgs e)
        {
            if (comboStocks.Text == "")
            {
                MessageBox.Show("Wybierz walor przed eksportem");
                return;
            }
            DataExport.ExportStockInfo();
        }

    }
}