using System;
using System.Collections.Generic;
using System.Text;

namespace AutoBroker
{
    public class StockStats
    {
        public int TotalFixings = 0;//computed

        public int StocksUpCount = 0;//computed
        public int StocksDownCount = 0;//computed

        public float MaxStockUpPercent = 0.0f;//computed
        public float MaxStockDownPercent = 0.0f;//computed

        public float AverageUpPercent = 0.0f;//computed
        public float AverageDownPercent = 0.0f;//computed

        public int LongestUpTrend = 0;//computed
        public int LongestDownTrend = 0;//computed

        public float AverageUpTrendLen = 0.0f;//computed
        public float AverageDownTrendLen = 0.0f;//computed

        public float[] AverageDayPercent = new float[7];//computed 0 = Sunday, 6 = Saturday

        public StockStats(List<StockValue> fixing)
        {
            if ((fixing == null)||(fixing.Count<2)) return;
            StockValue lastValue = fixing[0];
            TotalFixings = fixing.Count;
            MaxStockUpPercent = float.MinValue;
            MaxStockDownPercent = float.MaxValue;
            double totalUpPercents = 0.0f;
            double totalDownPercents = 0.0f;
            
            LongestUpTrend = 0;
            LongestDownTrend = 0;

            int currentUpTrendLen = 0;
            int currentDownTrendLen = 0;
            
            int totalUpTrendsLen = 0;
            int totalDownTrendsLen = 0;

            int totalUpTrends = 0;
            int totalDownTrends = 0;

            for (int i = 0; i < AverageDayPercent.Length; i++)
            {
                AverageDayPercent[i] = 0.0f;
            }
            int[] totalDayFixings = new int[7];
            
            for (int i=1;  i < fixing.Count; i++)
            {
                StockValue curValue = fixing[i];
                DateTime date = fixing[i].Date;                
                // adding a fake trend at the end change to compute averages
                bool trendChanged = (i == fixing.Count-1);
                float changeRate = curValue.CloseValue.ToFloat() / lastValue.CloseValue.ToFloat();

                totalDayFixings[(int)date.DayOfWeek]++;
                AverageDayPercent[(int)date.DayOfWeek] += changeRate;

                if (changeRate > MaxStockUpPercent) MaxStockUpPercent = changeRate;
                if (changeRate < MaxStockDownPercent) MaxStockDownPercent = changeRate;
                if (changeRate > 1.0f)
                {
                    totalUpPercents += changeRate;
                    StocksUpCount++;
                    if (currentDownTrendLen > 0) trendChanged = true;
                    currentUpTrendLen++;
                }
                if (changeRate < 1.0f)
                {
                    totalDownPercents += changeRate;
                    StocksDownCount++;
                    if (currentUpTrendLen > 0) trendChanged = true;
                    currentDownTrendLen++;
                }
                if (currentDownTrendLen > LongestDownTrend)
                {
                    LongestDownTrend = currentDownTrendLen;
                }
                if (currentUpTrendLen > LongestUpTrend)
                {
                    LongestUpTrend = currentUpTrendLen;
                }
                if (trendChanged)
                {
                    if (changeRate > 1.0f)
                    { // changing from - to +                        
                        totalDownTrendsLen += currentDownTrendLen;
                        totalDownTrends++;
                        currentDownTrendLen = 0;
                    }
                    else
                    { // changing from + to -
                        totalUpTrendsLen += currentUpTrendLen;
                        totalUpTrends++;
                        currentUpTrendLen = 0;
                    }
                }
                lastValue = curValue;
            }
            MaxStockUpPercent -= 1.0f;
            MaxStockDownPercent -= 1.0f;
            MaxStockDownPercent = Math.Abs(MaxStockDownPercent);

            AverageUpPercent = (float)(totalUpPercents / (double)StocksUpCount) - 1.0f;
            AverageDownPercent = Math.Abs((float)(totalDownPercents / (double)StocksDownCount) - 1.0f);

            if (totalUpTrends > 0)
            {
                AverageUpTrendLen = (float)totalUpTrendsLen / (float)totalUpTrends;
            }
            else
            {
                AverageUpTrendLen = 0;
            }

            if (totalDownTrends > 0)
            {
                AverageDownTrendLen = (float)totalDownTrendsLen / (float)totalDownTrends;
            }
            else
            {
                AverageDownTrendLen = 0;
            }

            for (int i = 0; i < AverageDayPercent.Length; i++)
            {
                if (totalDayFixings[i] > 0)
                {
                    AverageDayPercent[i] = ((AverageDayPercent[i] / (float)totalDayFixings[i]) - 1.0f);
                }
            }

        }

    }
}
