using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace AutoBroker
{
    public class Transaction
    {
        public DateTime Time = new DateTime();
        public string StockName = "Unknown";
        public int StockQuantityDelta = 0;
        public ValueRep SingleStockValue = new ValueRep(0.0f);
        public ValueRep Provision = new ValueRep(0.0f);        

        public static float ComputeNormalProvision(float totalPrice)
        {
            float prov = (float)Math.Abs(totalPrice * 0.0039f);
            if (prov < 5.0f)
            {
                prov = 5.0f;
            }
            return prov;
        }

        public static float ComputeTax(float worth)
        {
            if (worth < 0.0f) return 0.0f;
            return worth * 0.19f;
        }

        internal void WriteToStream(StreamWriter writer)
        {
            writer.Write(Time.ToShortDateString() + "|");
            writer.Write(StockName + "|");
            writer.Write(StockQuantityDelta.ToString() + "|");
            writer.Write(SingleStockValue.ToString()+"|");
            writer.WriteLine(Provision.ToString());

        }

        internal static Transaction FromStream(StreamReader reader)
        {
            string line = reader.ReadLine();
            string[] subLines = line.Split("|\n".ToCharArray());
            Transaction newTrans = new Transaction();
            newTrans.Time = DateTime.Parse(subLines[0]);
            newTrans.StockName = subLines[1];
            newTrans.StockQuantityDelta = int.Parse(subLines[2]);
            newTrans.SingleStockValue = ValueRep.Deserialize(subLines[3]);
            newTrans.Provision = ValueRep.Deserialize(subLines[4]);
            return newTrans;
        }
    }
}
