using System;
using System.Collections.Generic;
using System.Data;
using System.Text;

namespace AutoBroker
{
    public class AnyValues : IValues
    {
        private List<ValueRep> values = new List<ValueRep>();

        public int GetNumValues()
        {
            return values.Count;
        }

        public void AddValue(ValueRep anyValue)
        {
            values.Add(anyValue);
        }

        public float GetValue(int index)
        {
            return (values[index].ToFloat());
        }

        public int GetDecIntValue(int index)
        {
            return (values[index].ToDecInt());
        }

        public void GetMinMaxValue(out float min, out float max)
        {
            GetMinMaxValueForLast(values.Count, out min, out max);
        }

        public void GetMinMaxValueForLast(int lastCount, out float min, out float max)
        {
            min = new float();
            max = new float();
            if (values != null)
            {
                min = float.MaxValue;
                max = float.MinValue;
                for (int i = values.Count - 1; (i >= 0) && (lastCount > 0); i--, lastCount--)
                {
                    ValueRep curStock = values[i];
                    if (min > curStock.ToFloat()) min = curStock.ToFloat();
                    if (max < curStock.ToFloat()) max = curStock.ToFloat();
                }
            }
        }

        public void ShiftSequence(int rightShift)
        {
            if (rightShift == 0) return;
            int shift = Math.Abs(rightShift);
            List<ValueRep> nanList = new List<ValueRep>(shift);
            for (int i = 0; i < shift; i++)
            {
                nanList.Add(new ValueRep(float.NaN));
            }

            if (rightShift > 0)
            {
                values.InsertRange(0, nanList);
                values.RemoveRange(values.Count - shift, shift);
            }
            else
            {
                values.RemoveRange(0, shift);
                values.AddRange(nanList);
            }
        }

        public static AnyValues CreateFromIndicator(Indicator indicator)
        {
            AnyValues newValues = new AnyValues();
            for (int i=0; i< indicator.GetNumValues(); i++)
            {
                newValues.AddValue(new ValueRep(indicator.GetValue(i)));
            }
            return newValues;
        }

        public IValues Clone()
        {
            AnyValues newVals = new AnyValues();
            for (int i = 0; i < this.GetNumValues(); i++)
            {
                newVals.AddValue(new ValueRep(this.GetValue(i)));
            }
            return newVals;
        }

    }
}
